function [lab,info] = DisRFC(D,nclus,param)
% [lab,info] = DisRFC(D,nclus,param)
% Dissimilarity Random Forest Clustering
% 
% Dissimilarity Random Forest Clustering
% (c) Manuele Bicego 2020
% For all details please refer to
% M. Bicego: "Dissimilarity Random Forest Clustering", 
% Proc. Int. Conf. on Data Mining (ICDM2020)
%
% main routine for clustering
% 
% Input parameters:
% D     -> pairwise distance between points
% nclus -> number of clusters
% param -> parameters (if empty we use
%          param = DisRFC_defaultParam;) 
%
% Output:
% lab   -> the assigned labels
% info  -> structure with some information


if nargin < 3
    param = DisRFC_defaultParam;
end

% number of repetitions
Nrep = param.NrandInit;
stored_lab = cell(Nrep,1);
SilInd = zeros(Nrep,1);
for i = 1:Nrep
    fprintf('-- Repetition %d/%d\n',i,Nrep);
    % train Unsupervise Dissimilarity RF
    Rf = DisRFC_trainUDRF(D,param);
    % compute the embedding
    emb = DisRFC_embed(D,Rf,param);
    % perform the clustering
    lab = DisRFC_clust(emb,nclus,param);
    stored_lab{i} = lab;
    % Compute silhouette index
    SilInd(i) = distSilhouette(D,lab,nclus);
end
% best run according to Silhouette index
[a,b] = max(SilInd);
lab = stored_lab{b};
info.bestSI = a;
info.stored_lab = stored_lab;
info.SilInd = SilInd;
info.param = param;