function [T,errf] = DisRFC_KCC(sig,mtd,nclus,nrep,verbose)
% [T,errf] = DisRFC_KCC(sig,mtd,nclus,nrep,verbose)
% 
% K-means Based Consensus Clustering from
% Wu et al: "K-means Based Consensus Clustering; a unified view",
% IEEE Tr. on TKDD, vol 27(1), 2015
% 
% Different KCC utility functions (see tab 3 of the paper)
% mtd = 1 -> KCC-Uc 
% mtd = 2 -> KCC-UH
% mtd = 3 -> KCC-Ucos
% mtd = 4 -> KCC-UL5
% mtd = 5 -> KCC-UL8
%
% In this version all trees have the same importance
% i.e. weights are all equals to 1/nT


% First step: creating Xb
[nO,nT] = size(sig);
csig = num2cell(sig,1);
cndif = cellfun(@unique,csig,'UniformOutput',false);
mx = cellfun(@length,cndif);
Nsig = zeros(nO,sum(mx));
ini = 1;
eni = mx(1);
for t = 1:nT
    pi = sig(:,t);
    xu = unique(pi);
    pi2 = zeros(size(pi));
    for j = 1:length(xu)
        pi2(pi==xu(j)) = j;
    end
    a = sub2ind([nO,mx(t)],[1:nO]',pi2);
    b = zeros(nO,max(pi2));
    b(a) = 1;
    Nsig(:,ini:eni) = b;
    if t<nT
        ini = ini+mx(t);
        eni = eni+mx(t+1);
    end
end
dim = size(Nsig,2);

% second step: k-means
[T,errf] = DisRFC_KCC_Kmeans_fast2(Nsig,nT,mx,mtd,nclus,nrep,verbose);


