function [T,errf] = DisRFC_KCC_Kmeans_fast2(Nsig,nT,mx,mtd,nclus,Nrep,verbose)
% [T,errf] = DisRFC_KCC_Kmeans_fast2(Nsig,nT,mx,mtd,nclus,Nrep,verbose)
%
% Implements the kmeans needed for KCC
% K-means Based Consensus Clustering
% Wu et al: "K-means Based Consensus Clustering; a unified view",
% IEEE Tr. on TKDD, vol 27(1), 2015
%
% Different KCC utility functions (see tab 3 of the paper)
% mtd = 1 -> KCC-Uc
% mtd = 2 -> KCC-UH
% mtd = 3 -> KCC-Ucos
% mtd = 4 -> KCC-UL5
% mtd = 5 -> KCC-UL8
%


[n,d] = size(Nsig);


if mtd == 3 || mtd == 4 || mtd == 5
    CNsig = mat2cell(Nsig,ones(1,n),mx);
elseif mtd == 2
    %Nsig = Nsig./(repmat(sum(Nsig),[n,1]));
    XNsig = Nsig;
    XNsig(XNsig==0) = eps;
    PNsig = XNsig./repmat(sum(XNsig,2),[1 d]);
end
for esp = 1:Nrep
    % init
    labels = randi(nclus,[n 1]);
    
    % main loop
    iter = 0;
    mk = zeros(nclus,d);
    clear f
    while true
        iter = iter+1;
        
        % mean assignment
        for k=1:nclus
            if sum(labels==k)
                mk(k,:) = mean(Nsig(labels==k,:));
            end
        end
        
        Xmk = mk;
        Xmk(Xmk==0) = eps;
        Pmk = Xmk./repmat(sum(Xmk,2),[1 d]);
        
        %         PNsig = Nsig./repmat(sum(Nsig,2),[1 d]);
        %         Pmk = mk./repmat(sum(mk,2),[1 d]);
        %         % compute distance
        dd = zeros(n,nclus);
        if mtd == 3 || mtd ==4 || mtd == 5
            xmk = mat2cell(mk,ones(1,nclus),mx);
        end
        for k = 1:nclus
            switch mtd
                case 1 % KCC-Uc
                    a = ((Nsig - repmat(mk(k,:),[n,1])).^2);
                    dd(:,k) = 1/nT*sum(a,2);
                case 2 % KCC-UH
                    % KL(xi,yi) = sum xi log xi/yi
                    % xi,yi should sum to 1
                    xbi = PNsig;
                    ybi = repmat(Pmk(k,:),[n,1]);
                    a = xbi.*log2(xbi./ybi);
                    dd(:,k) = 1/nT*sum(a,2);
                case 3 % KCC-Ucos
                    axbi = CNsig;
                    aybi = repmat(xmk(k,:),[n 1]);
                    a1 = cellfun(@cosdist,axbi,aybi);
                    dd(:,k) =  1/nT*sum(a1,2);
                case {4,5} % KCC-ULp
                    % parameters from Wu et al
                    axbi = CNsig;
                    aybi = repmat(xmk(k,:),[n 1]);
                    if mtd == 4 % p = 5
                        % parameters from Wu et al
                        a1 = cellfun(@Lp5dist,axbi,aybi);
                    else % p = 8
                        a1 = cellfun(@Lp8dist,axbi,aybi);
                    end
                    dd(:,k) =  1/nT*sum(a1,2);
        
                    
            end
        end
        % find nearest prototype
        old_labels = labels;
        [~,labels] = min(dd,[],2);
        %Check convergence
        if labels == old_labels
            break
        end
        if iter > 100
            break;
        end
        f(iter) = 0;
        for k = 1:nclus
            f(iter) = f(iter) + sum(dd(labels==k,k));
        end
    end    
    % compute optimized function
    %   plot(f),pause
    minf(esp) = 0;
    for k = 1:nclus
        minf(esp) = minf(esp) + sum(dd(labels==k,k));
    end
    Xlabels{esp} = labels;
    if verbose, fprintf('Iter %d - minf %.3f\n',esp,minf(esp)), end
end
% find best conf
[errf,a] = min(minf);
T = Xlabels{a};


