function [T,errf ] = DisRFC_kmodes(x,nclus,nrep)
% [T,errf ] = DisRFCkmodes(x,nclus,nrep)
% 
% Kmodes algorithm for categorical data clustering
% as in ZHEXUE HUANG: "Extensions to the k-Means Algorithm for Clustering
% Large Data Sets with Categorical Values",
% Data Mining and Knowledge Discovery 2, 283--304 (1998)
% 
% T -> clustering result
% errf -> value of minimized function

[n,d] = size(x);
Xdtot = zeros(1,nrep);
Xlabels = zeros(n,nrep);
Xiter = zeros(1,nrep);

for esp = 1:nrep
    % init 
    labels = randi(nclus,[n 1]);
    iter = 0;
    R = zeros(nclus,d);
    while true
        iter = iter+1;
        
        % compute representatives
        for k=1:nclus
            R(k,:) = mode(x(labels==k,:));
        end
        % compute distances from clustering
        dd = zeros(n,k);
        for k=1:nclus
            dd(:,k) = sum(x ~= repmat(R(k,:),[n 1]),2);
        end
        % label assignment
        old_labels = labels;
        [~,labels] = min(dd,[],2);
        
        %Check for convergence
        if labels == old_labels
            break
        end
    end
    % compute error
    dtot = 0;
    for k=1:nclus
        ndx = find(labels==k);
        dtot = dtot + sum(sum(x(ndx,:) ~= repmat(R(k,:),[length(ndx) 1]),2));
    end
    Xdtot(esp) = dtot;
    Xlabels(:,esp) = labels;
    Xiter(esp) = iter;
end
[errf,b] = min(Xdtot);
T = Xlabels(:,b);

