function rft = DisRFC_trainUDRF(D,param)
% rft = DisRFC_trainUDRF(D,param)
% Train an Unsupervised Dissimilarity Random Forest
%
% Dissimilarity Random Forest Clustering
% (c) Manuele Bicego 2020
% For all details please refer to
% M. Bicego: "Dissimilarity Random Forest Clustering", 
% Proc. Int. Conf. on Data Mining (ICDM2020)
%
% Input parameters:
% D     -> pairwise distance between points
% param -> parameters
%          For standard parameter simply use
%          param = DisRFC_defaultParam;
% 
% Output:
% rft    -> the trained UD-RF

if nargin < 2
    param = DisRFC_defaultParam;
end

% compute nearest neighbors
[~,NNs] = sort(D,2);

n = size(D,1);
objs = 1:n;

if param.maxDepth == 0
    % automatic depth
    param.maxDepth = ceil(log2(n));
end

if param.sampling<=1
    % fraction
    psi = round(n*param.sampling);
else
    % number
    psi = param.sampling;
    if psi> n
        psi = n;
    end
end
param.renyidivK = round(sqrt(psi)); 
param.maxDepthSigFull = round(log2(psi));
global lid 
if param.verbose, fprintf('Training  (UDRF with %d trees): ',param.ntrees), end
for i = 1:param.ntrees
    % step 1: random sampling of a subset
    aa = randperm(n);
    bt = aa(1:psi);
    btobjs = objs(bt);
    
    if length(btobjs) < param.minsizeSplit
        % too few objects, force at least one split
        param.minsizeSplit = round(length(btobjs)/2);
    end
    % step 2: train tree
    if param.verbose
        if (mod(i,10) == 0) || (i == 1) || (i == param.ntrees),fprintf('.');end; 
    end
    lid = 1;
    t1 = DisRFC_trainUDT(D,NNs,btobjs,param,1);
    rft.trees{i} = t1;
    rft.dep(i) = t1.maxdepth;
end
if param.verbose, fprintf('\n');end
rft.param = param;    
