function EVA = distSilhouette(DD,T,nclus)
% EVA = distSilhouette(DD,T,nclus)
%
% compute silhouette index using distances in DD and labels in T
% P. Rousseeuw, “Silhouettes: A graphical aid to the interpretation and
% validation of cluster analysis,” J. Comput. Appl. Math., 20, 53–65, 1987.


if ~issym(DD)
    fprintf('Silhouette: Distance not symmetric\n');
    DD = (DD+DD')/2;
end

nel = size(DD,1);

if length(unique(T)) > nclus
    T(T>nclus) = nclus;
    fprintf('Too many labels\n');
end

s = zeros(nel,1);
for i = 1:nel
    ndx = find(T == T(i)); % objects in A (cluster of i)
    if length(ndx) == 1
        % only one object
        s(i) = 0;
    else
        ndx(ndx == i) = []; % remove i itself
        ai = mean(DD(i,ndx));
        d = zeros(nclus,1);
        for c = 1:nclus
            if c ~= T(i)
                ndx = (T == c); % objects in C different from A
                d(c) = mean(DD(i,ndx));
            end
        end
        d(T(i)) = []; % remove A
        bi = min(d);
        s(i) = (bi-ai)./(max(ai,bi));
    end
end
EVA = mean(s);




