% Demo for Dissimilarity Random Forest Clustering
%
% Dissimilarity Random Forest Clustering
% (c) Manuele Bicego 2020
% For all details please refer to
% M. Bicego: "Dissimilarity Random Forest Clustering", 
% Proc. Int. Conf. on Data Mining (ICDM2020)
%

clear all
close all


fprintf(' -- Demo for Dissimilarity Random Forest Clustering\n');
fprintf(' -- \n');
fprintf(' -- For all details please refer to\n');
fprintf(' -- M. Bicego: "Dissimilarity Random Forest Clustering",\n'); 
fprintf(' -- Proc. Int. Conf. on Data Mining (ICDM2020)\n');



% compute/load your distance

% Here a demo file from PRDisData:
% http://37steps.com/distools/
load('Dist_demo.mat');
% D      -> the pairwise dissimilarities
% labels -> the true labels
% 
% D contains a 213x213 dissimilarity matrix comparing 
% the protein sequences based on the concept of an evolutionary distance. 
% There are four classes of globins: heterogeneous 
% globin (G), hemoglobin-A (HA), hemoglobin-B (HB) and myoglobin (M).
%
% REFERENCES
% T. Graepel, R. Herbrich, P. Bollmann-Sdorra, K. Obermayer, Classification 
% on pairwise proximity data. In Adv. in Neural Information System Processing 
% vol. 11, 438-444, 1999.





% parameters for DisRFC
% to use default parameters use:
param = DisRFC_defaultParam;

% param.UDT - type of Uns. Diss. Trees
  % 1: (default) Rand
  % 2: HausD 
  % 3: RenyiD
% param.emb - type of Embedding with UD-RF
  % 1: (default) One Partition (leaves level)
  % 2: Multiple Partitions
% param.MPparam % partitions in MP 
  % default: 3
% param.ensC - type of ensemble clustering
  % 1: (default) KM-SED: Kmeans with Squared Euclidean Distance
  % 2: KM-KL:  K-means with KL divergence
  % 3: K-medoids: K-medoids
  % 4: K-modes: K-modes
% param.ntrees - number of trees
  % default: 100;
% param.maxDepth - max depth
  % default: inf;
% param.minsizeSplit - min number of objects in a node
                     % to perform a split
  % default: 10
% param.sampling - objects to sample to build each tree
                 % if less than 1 -> the fraction of objects used to build each tree
                 % if larger than 1 -> the number of objects used to build each tree
  % default: 128
% param.NrandInit - number of random repetitions
  % default: 10 
% param.nrandKM - repetitions for all kmeans methods 
  % default: 20 
% param.maxTrialsSplit - maximum number of random splits among
                         % which chosing the best pair for splitting
  % default: 20
% param.renyidivAlpha - alpha for the Renyi Divergence
  % default: 0.9999
% param.verbose = verbose mode 
  % default = 1;


% Changing training to Renyii criterion
% param.UDT = 3;
    
nclus = length(unique(labels));

% Calling the main routine
[lab,info] = DisRFC(D,nclus,param);

% Check the results
ARI = RandIndex(lab,labels);

fprintf('Adjusted Rand Index: %.4f\n',ARI);


